﻿-- ====================================================================
--  CL3.lua
--  Contains code for performing operations on list data
-- ====================================================================
local Torta = Torta;
local Checklist3 = Torta.cl3;
local gui = Torta.gui;
local widgets = Torta.gui.elements;
local displayWindowPoints = { widgets.clDisplayWindow:GetPoint(1) };
local dispWnd = widgets.clDisplayWindow;
local tEditor = widgets.taskEditor;

local newListCount = 0;

function Checklist3:InitializeChecklist()
  if type(Torta.db.profile.cfg.loadedList) == "number" then
    self:LoadList(Torta.db.profile.cfg.loadedList);
  else
    self:LoadList(nil)
  end
  
  self:ListLoadedCheck();
  
  if not self:IsEditTaskDialogOpen() then
    widgets.removeTaskButton:Disable();
  end
end


function Checklist3:LoadList(listID)
  local lists = Torta.db.profile.lists;
  self:HideTaskEditor();
  if (listID == nil) or (Torta.gui.listData == lists[listID]) then
    -- "Unload" the list, show the no list loaded text, hide stuff
    gui.listData = nil;
    
    widgets.cl3HeaderText:SetText("Torta Checklist");
    Torta.db.profile.cfg.loadedList = false;
    self.openTask = nil;
  else
    -- Do the opposite!
    gui.listData = lists[listID];
    FauxScrollFrame_SetOffset(Torta.gui.elements.clScrollBar, 0);
    getglobal(widgets.clScrollBar:GetName() .. "ScrollBar"):SetValue(0);
    widgets.cl3HeaderText:SetText(lists[listID].meta.listName);
    Torta.db.profile.cfg.loadedList = listID;
    self.openTask = nil;
  end
  gui:UpdateCLScrollBar();
  gui:UpdateExplorerScrollBar();
  self:ListLoadedCheck()
end


function Checklist3:GetLoadedList()
  return Torta.db.profile.lists[Torta.db.profile.cfg.loadedList];
end


function Checklist3:GetList(listID)
  return Torta.db.profile.lists[listID];
end


function Checklist3:ListLoadedCheck()
  if not self:GetLoadedList() then
    widgets.checkAllButton:Disable();
    widgets.uncheckAllButton:Disable();
    widgets.addTaskButton:Disable();
  else
    widgets.checkAllButton:Enable();
    widgets.uncheckAllButton:Enable();
    widgets.addTaskButton:Enable();
  end
end


function Checklist3:EditTask(taskID)
  if self.openTask and self.openTask == taskID then
    -- close the task editor
    self.openTask = nil;
    self:HideTaskEditor();
  else
    -- open the task editor
    self:ShowTaskEditor(taskID);
    widgets.editTaskEB:SetFocus();
    self.openTask = taskID;
    widgets.editTaskEB:SetText(self:GetLoadedList().text[taskID]);
  end
  gui:UpdateCLScrollBar();
end


function Checklist3:IsEditTaskDialogOpen()
  return not not self.openTask;
end


function Checklist3:ModifyOpenTask(newText)
  assert(type(self.openTask) == "number");
  local profile = Torta.db.profile;
  self:GetLoadedList().text[self.openTask] = newText;
end


function Checklist3:HideTaskEditor()
  dispWnd:ClearAllPoints();
  dispWnd:SetPoint(unpack(displayWindowPoints));
  dispWnd:SetPoint("BOTTOMRIGHT", widgets.clTaskBar, "TOPRIGHT");
  tEditor:Hide();
  self.openTask = nil;
  widgets.removeTaskButton:Disable();
  gui:UpdateCLScrollBar();
end


function Checklist3:ShowTaskEditor(taskID)
  tEditor:Show();
  dispWnd:ClearAllPoints();
  dispWnd:SetPoint(unpack(displayWindowPoints));
  dispWnd:SetPoint("BOTTOMRIGHT", tEditor, "TOPRIGHT");
  getglobal(tEditor:GetName() .. "HeaderText"):SetText("Task Editor: " .. (taskID or "|cffff0000No task ID|r"))
  widgets.removeTaskButton:Enable();
  gui:CheckCLButtons();
  gui:MakeTaskVisible(taskID);
  gui:UpdateCLScrollBar();
end


function Checklist3:UncheckAllTasks()
  local list = self:GetLoadedList();
  for i = 1, #list.stat do
    list.stat[i] = false;
  end
  gui:UpdateCLScrollBar();
end

function Checklist3:CheckAllTasks()
  local list = self:GetLoadedList();
  for i = 1, #list.stat do
    list.stat[i] = true;
  end
  gui:UpdateCLScrollBar();
end


function Checklist3:OnCheckClicked(id)
  local list = self:GetLoadedList();
  if list then
    list.stat[id] = not list.stat[id];
  end
  gui:UpdateCLScrollBar();
end


function Checklist3:AddTask()
  local list = self:GetLoadedList();
  if not list then return end
  
  local index = (self.openTask or #list.text) + 1;
  tinsert(list.text, index, "");
  tinsert(list.stat, index, false);
  gui:UpdateCLScrollBar();
end

function Checklist3:RemoveTask()
  local list = self:GetLoadedList();
  if not list then return end
  
  tremove(list.text, self.openTask);
  tremove(list.stat, self.openTask);
  gui:UpdateCLScrollBar();
  self:HideTaskEditor();
end


function Checklist3:CreateList()
  newListCount = newListCount + 1;
  tinsert(Torta.db.profile.lists, {
    meta = {
      listName = "New List " .. newListCount
    },
    text = {},
    stat = {}
  });
  gui:UpdateExplorerScrollBar();
end

function Checklist3:DeleteList(index, confirmed)
  assert(index);
  if confirmed then
    local loadedListIndex = Torta.db.profile.cfg.loadedList;
    tremove(Torta.db.profile.lists, index);
    -- print(index, loadedListIndex);
    if loadedListIndex then
      if index < loadedListIndex then
        -- print(index, loadedListIndex, index < loadedListIndex);
        self:LoadList(nil);
        self:LoadList(loadedListIndex - 1);
      end
    end
    gui:UpdateExplorerScrollBar();
    if not Torta.db.profile.lists[Torta.db.profile.cfg.loadedList] then
      -- print(Torta.db.profile.cfg.loadedList);
      Torta.db.profile.cfg.loadedList = false;
      self:LoadList(nil);
      gui:UpdateCLScrollBar();
    end
  else
    local dialog = StaticPopup_Show("TORTA_CLDELETELIST", self:GetList(index).meta.listName)
    if dialog then
      dialog.data = index;
    end
  end
end

function Checklist3:RenameList(listID, newName)
  -- print(listID, newName);
  if not (type(newName) == "string") then
    local listName = self:GetList(listID).meta.listName
    local dialog = StaticPopup_Show("TORTA_CLRENAMELIST", listName);
    getglobal(dialog:GetName() .. "EditBox"):SetText(listName);
  else
    Torta.db.profile.lists[listID].meta.listName = newName;
    gui:UpdateExplorerScrollBar();
    gui:UpdateCLScrollBar();
  end
end

function Checklist3:RestoreList()
  
end

function Checklist3:UpdateStats()
  local checked, total, percent = 0, 0, 0;
  local list = self:GetLoadedList();
  if list then
    total = #list.stat;
    if total == 0 then
      widgets.statsButton:SetText("");
      return;
    end
    for i = 1, total do
      if list.stat[i] then
        checked = checked + 1;
      end
    end
    percent = format("%.1f", (checked / total) * 100);
    widgets.statsButton:SetFormattedText("Tasks: %d/%d (%s)", checked, total, percent .. "%");
  else
    widgets.statsButton:SetText("");
  end
end

-- Drop handler functions
function Checklist3:MoveTask(SrcID, DestID)
  local list = self:GetLoadedList();
  local text = list.text[SrcID];
  local stat = list.stat[SrcID];
  
  if DestID > #list.text then -- If the button was dragged to be below the last item
    DestID = #list.text;
  end
  
  tinsert(list.text, DestID, text);
  tinsert(list.stat, DestID, stat);
  
  if DestID < SrcID then
    SrcID = SrcID + 1;
  end
  
  tremove(list.text, SrcID);
  tremove(list.stat, SrcID);
  
  gui:UpdateCLScrollBar();
end

function Checklist3:MoveList(SrcID, DestID)
  local listsTbl = Torta.db.profile.lists;
  local list = listsTbl[SrcID];
  
  if DestID > #listsTbl then -- If the button was dragged to be below the last item
    DestID = #listsTbl;
  end
  
  tinsert(listsTbl, DestID, list);
  
  if DestID < SrcID then
    SrcID = SrcID + 1;
  end
  
  tremove(listsTbl, SrcID);
  
  gui:UpdateExplorerScrollBar();
end
