﻿-- ====================================================================
--  UIManager.lua
--  Contains functions used for operations that impact what the user
--  sees.
-- ====================================================================

-- Windows
local                 mainWindow = TortaParentFrame;
local                  startPage = TortaParentFrameStartPage;
local                   mainPage = TortaParentFrameMainPage;
local             explorerWindow = TortaParentFrameMainPageExplorerSubWindow;
local            clDisplayWindow = TortaParentFrameMainPageCLDisplaySubWindow;
local            nbDisplayWindow = TortaParentFrameMainPageNBDisplaySubWindow;

-- Misc. Widgets
local          explorerScrollBar = TortaParentFrameMainPageExplorerSubWindowScrollBar;
local                clScrollBar = TortaParentFrameMainPageCLDisplaySubWindowScrollBar;
local           noLoadedListText = TortaParentFrameMainPageCLDisplaySubWindowNoLoadedListText;
local                noTasksText = TortaParentFrameMainPageCLDisplaySubWindowNoTasksText;
local                clTopButton = TortaParentFrameMainPageCLDisplaySubWindowTaskButton1;
local              cl3HeaderText = TortaParentFrameMainPageCLDisplaySubWindowHeaderText;
local                 taskEditor = TortaParentFrameMainPageTaskEditorSubWindow;
local                  clTaskBar = TortaParentFrameMainPageCLTaskbar;
local                  nbTaskBar = TortaParentFrameMainPageNBTaskbar;
local                 editTaskEB = TortaParentFrameMainPageTaskEditorSubWindowScrollFrameEditBox;
local           uncheckAllButton = TortaParentFrameMainPageCLTaskbarUncheckAllButton;
local             checkAllButton = TortaParentFrameMainPageCLTaskbarCheckAllButton;
local              addTaskButton = TortaParentFrameMainPageCLTaskbarAddTaskButton;
local           removeTaskButton = TortaParentFrameMainPageCLTaskbarRemoveTaskButton;
local        explorerNoListsText = TortaParentFrameMainPageExplorerSubWindowNoListsText;
local        explorerNoNtbksText = TortaParentFrameMainPageExplorerSubWindowNoNotebooksText;
local                statsButton = TortaParentFrameMainPageCLTaskbarStatsButton;

local EXPLORER_LISTBUTTON_HEIGHT = 16;
local Torta = Torta;

noLoadedListText:SetText("|cffffd000No loaded list. I feel ignored. /cry|r\nYou currently do not have a list loaded. You can either select a list in Turtle Explorer or create a new one by right-clicking inside Turtle Explorer.");

noTasksText:SetText("|cffffd000Woah, no tasks!|r\nThe currently loaded list does not appear to have any tasks in it. You can create new tasks for this list by clicking the \"Add A Task\" button below.");

-- Explorer subwindow stuff
Torta.gui.exRecycleBin = {};
Torta.gui.exListButtons = { TortaParentFrameMainPageExplorerSubWindowListButton1 };
-- Checklist subwindow stuff
Torta.gui.clRecycleBin = {};
Torta.gui.clListButtons = { TortaParentFrameMainPageCLDisplaySubWindowTaskButton1 };

-- ====================================================================
--  Allow public access to our "shortcuts"
-- ====================================================================
Torta.gui.elements = {};
local e = Torta.gui.elements;
e.mainWindow          = mainWindow;
e.startPage           = startPage;
e.explorerWindow      = explorerWindow;
e.explorerScrollBar   = explorerScrollBar;
e.clDisplayWindow     = clDisplayWindow;
e.clScrollBar         = clScrollBar;
e.noLoadedListText    = noLoadedListText;
e.clTopButton         = clTopButton;
e.cl3HeaderText       = cl3HeaderText;
e.taskEditor          = taskEditor;
e.clTaskBar           = clTaskBar;
e.nbTaskBar           = nbTaskBar;
e.editTaskEB          = editTaskEB;
e.uncheckAllButton    = uncheckAllButton;
e.checkAllButton      = checkAllButton;
e.addTaskButton       = addTaskButton;
e.removeTaskButton    = removeTaskButton;
e.explorerNoListsText = explorerNoListsText;
e.explorerNoNtbksText = explorerNoNtbksText
e.statsButton         = statsButton;
e = nil;
-- ====================================================================

function Torta.gui.scalingFinishedFunc(frame)
  frame:Hide();
end

local FADETIME = 0.25;
function Torta.gui:ToggleWindow(window)
  window = window or mainWindow;
  if self:IsFrameScaling(window) or UIFrameIsFading(window) then
    return;
  end
  if not window:IsVisible() then
    window.scalingFinishedFunc = nil;
    window:Show();
    self:ScaleIn(window, FADETIME, 0.8, 1);
    UIFrameFadeIn(window, FADETIME, 0, 1);
  else
    window.scalingFinishedFunc = self.scalingFinishedFunc;
    self:ScaleOut(window, FADETIME, 1, 0.7);
    UIFrameFadeOut(window, FADETIME, 1, 0);
  end
end


function Torta.gui:OnTabClicked(tabID)
  assert(type(tabID) == "number");
  PanelTemplates_SetTab(mainWindow, tabID);
  if Torta.cl3.HideTaskEditor then
    Torta.cl3:HideTaskEditor();
  end
  if tabID == 1 then
    startPage:Show();
    mainPage:Hide();
  else
    self.tab = tabID;
    startPage:Hide();
    mainPage:Show();
    self:UpdateExplorerScrollBar();
    if tabID == 2 then
      clDisplayWindow:Show();
      clTaskBar:Show();
      
      nbDisplayWindow:Hide();
      nbTaskBar:Hide();
    else
      clDisplayWindow:Hide();
      clTaskBar:Hide();
      
      nbDisplayWindow:Show();
      nbTaskBar:Show();
    end
  end
end


function Torta.gui:Initialize()
  
end

-- Explorer listbox code
function Torta.gui:CheckExplorerWindow()
  local numButtonsCanFit = floor((explorerWindow:GetHeight() - (20 + 2 + 2)) / EXPLORER_LISTBUTTON_HEIGHT);
  local numButtons = #self.exListButtons
  if numButtonsCanFit == numButtons then
    return;
  end
  
  if numButtonsCanFit > numButtons then
    self:CreateExplorerButtons(numButtonsCanFit - numButtons);
  else
    self:RemoveExplorerButtons(numButtons - numButtonsCanFit);
  end
end

function Torta.gui:CreateExplorerButtons(num)
  for i = 1, num do
    local id = #self.exListButtons + 1;
    local button = (self.exRecycleBin[id] or CreateFrame(
        "Button",
        nil,
        explorerWindow,
        "TortaExplorerButtonTemplate"));
    button:ClearAllPoints();
    button:SetPoint("TOPLEFT", self.exListButtons[id - 1], "BOTTOMLEFT");
    button:SetPoint("TOPRIGHT", self.exListButtons[id - 1], "BOTTOMRIGHT");
    button:SetID(id);
    button:Show();
    self.exListButtons[id] = button;
  end
  self:UpdateExplorerScrollBar();
end

function Torta.gui:RemoveExplorerButtons(num)
  for i = 1, num do
    local id = #self.exListButtons
    self.exRecycleBin[id] = self.exListButtons[id];
    self.exListButtons[id]:Hide();
    self.exListButtons[id] = nil;
  end
  self:UpdateExplorerScrollBar();
end


function Torta.gui:UpdateExplorerScrollBar()
  self = Torta.gui;
  local lineplusoffset;
  local button;
  explorerNoListsText:Hide();
  explorerNoNtbksText:Hide();
  
  if self.tab == 2 then -- if Torta Checklist is selected
    if #Torta.db.profile.lists > 0 then
      local lists = Torta.db.profile.lists;
      FauxScrollFrame_Update(explorerScrollBar, #Torta.db.profile.lists, #self.exListButtons, 16, nil, nil, nil, nil, nil, nil, true)
      local loadedListID = Torta.db.profile.cfg.loadedList;
      for line = 1, #self.exListButtons do
        lineplusoffset = line + FauxScrollFrame_GetOffset(explorerScrollBar);
        button = self.exListButtons[line];
        if lineplusoffset <= #Torta.db.profile.lists then
          if not lists[lineplusoffset] then return end
          button.text:SetText(lists[lineplusoffset].meta.listName);
          button:SetID(lineplusoffset);
          
          if lineplusoffset ~= loadedListID then
            button:UnlockHighlight();
          else
            button:LockHighlight();
          end
          
          button.topDropHandler.destLoc = lineplusoffset;
          button.bottomDropHandler.destLoc = lineplusoffset + 1;
          
          button:Show();
        else
          button:Hide();
        end
      end
    else
      explorerScrollBar:Hide();
      explorerNoListsText:Show();
      for i = 1, #self.exListButtons do
        self.exListButtons[i]:Hide();
      end
    end
  else  -- if Torta Notebook is selected
    if #Torta.db.profile.notes > 0 then
      local notes = Torta.db.profile.notes;
      FauxScrollFrame_Update(explorerScrollBar, #Torta.db.profile.notes, #self.exListButtons, 16, nil, nil, nil, nil, nil, nil, true)
      local loadedListID = Torta.db.profile.cfg.loadedList;
      for line = 1, #self.exListButtons do
        lineplusoffset = line + FauxScrollFrame_GetOffset(explorerScrollBar);
        button = self.exListButtons[line];
        if lineplusoffset <= #Torta.db.profile.notes then
          if not notes[lineplusoffset] then return end
          button:SetText(notes[lineplusoffset].meta.name);
          button:SetID(lineplusoffset);
          
          if lineplusoffset ~= loadedListID then
            button:UnlockHighlight();
          else
            button:LockHighlight();
          end
          
          button:Show();
        else
          button:Hide();
        end
      end
    else
      explorerScrollBar:Hide();
      explorerNoNtbksText:Show();
      for i = 1, #self.exListButtons do
        self.exListButtons[i]:Hide();
      end
    end
  end
end



function Torta.gui:CheckCLButtons()
  local numButtonsCanFit = floor((clDisplayWindow:GetHeight() - (20 + 2 + 2)) / 32);
  local numButtons = #self.clListButtons
  if numButtonsCanFit == numButtons then
    return;
  end
  
  if numButtonsCanFit > numButtons then
    self:CreateCLButtons(numButtonsCanFit - numButtons);
  else
    self:RemoveCLButtons(numButtons - numButtonsCanFit);
  end
end

function Torta.gui:CreateCLButtons(num)
  for i = 1, num do
    local id = #self.clListButtons + 1;
    local button = (self.clRecycleBin[id] or CreateFrame(
        "Button",
        nil,
        clDisplayWindow,
        "TortaChecklistTaskButton"));
    button:ClearAllPoints();
    button:SetPoint("TOPLEFT", self.clListButtons[id - 1], "BOTTOMLEFT");
    button:SetPoint("TOPRIGHT", self.clListButtons[id - 1], "BOTTOMRIGHT");
    button:SetID(id);
    button:Show();
    self.clListButtons[id] = button;
  end
  self:UpdateCLScrollBar();
end

function Torta.gui:RemoveCLButtons(num)
  for i = 1, num do
    local id = #self.clListButtons
    self.clRecycleBin[id] = self.clListButtons[id];
    self.clListButtons[id]:Hide();
    self.clListButtons[id] = nil;
  end
  self:UpdateCLScrollBar();
end

-- CHECKLIST DISPLAY FUNCTION
function Torta.gui:UpdateCLScrollBar(isFromOnShow)
  self = Torta.gui;
  local lineplusoffset;
  local button;
  
  noTasksText:Hide();
  noLoadedListText:Hide();
  
  clDisplayWindow:Show();
  nbDisplayWindow:Hide();
  if self.listData then
    noLoadedListText:Hide();
    cl3HeaderText:SetText(self.listData.meta.listName);
    FauxScrollFrame_Update(clScrollBar, #self.listData.text, #self.clListButtons, 32, nil, nil, nil, nil, nil, nil, true)
    if not isFromOnShow then
      clScrollBar:Show();
    end
    for line = 1, #self.clListButtons do
      lineplusoffset = line + FauxScrollFrame_GetOffset(clScrollBar);
      button = self.clListButtons[line];
      if lineplusoffset <= #self.listData.text then
        button.text:SetFormattedText("|cffffd000%i.)|r %s", lineplusoffset, self.listData.text[lineplusoffset]);
        button.check:SetChecked(self.listData.stat[lineplusoffset]);
        button:SetID(lineplusoffset);
        button.check:SetID(lineplusoffset);
        button:SetID(lineplusoffset);
        
        local topText = "Insert above task " .. lineplusoffset;
        local btmText = "Insert below task " .. lineplusoffset;
        
        button.topDropHandler:SetFormattedText("%s - %s - %s", topText, topText, topText);
        button.topDropHandler.destLoc = lineplusoffset;
        button.bottomDropHandler:SetFormattedText("%s - %s - %s", btmText, btmText, btmText);
        button.bottomDropHandler.destLoc = lineplusoffset + 1;
        
        if lineplusoffset ~= Torta.cl3.openTask then
          button:UnlockHighlight();
        else
          button:LockHighlight();
        end
        
        button:Show();
      else
        button:Hide();
      end
    end
    if #self.listData.text == 0 then
      noTasksText:Show();
      clScrollBar:Hide()
    else
      noTasksText:Hide();
      clScrollBar:Show();
    end
  else
    noLoadedListText:Show();
    clScrollBar:Hide();
    for line = 1, #self.clListButtons do
      self.clListButtons[line]:Hide();
    end
  end
  Torta.cl3:UpdateStats();
end


function Torta.gui:OnManageDialogShow()
  if Torta.cl3:IsEditTaskDialogOpen() then
    taskEditor:Hide();
  end
  clDisplayWindow:Hide();
end

function Torta.gui:OnManageDialogHide()
  if Torta.cl3:IsEditTaskDialogOpen() then
    taskEditor:Show();
  end
  clDisplayWindow:Show();
end


function Torta.gui:IsTaskVisible(id)
  for k,button in pairs(self.clListButtons) do
    if button:GetID() == id then
      return true;
    end
  end
  return false;
end


function Torta.gui:MakeTaskVisible(taskID)
  assert(type(taskID) == "number", "Assertion failed: method MakeTaskVisible requires a number parameter (taskID)");
  local scrollBar = getglobal(clScrollBar:GetName() .. "ScrollBar");
  local offset = FauxScrollFrame_GetOffset(clScrollBar);
  for i = 1, 1000 do  -- just so we don't go into an infinite loop
    if self:IsTaskVisible(taskID) then
      break;
    end
    
    FauxScrollFrame_SetOffset(clScrollBar, offset);
    FauxScrollFrame_Update(clScrollBar, #Torta.cl3:GetLoadedList().text, #self.clListButtons, 32, nil, nil, nil, nil, nil, nil, true)
    self:CheckCLButtons();
    self:UpdateCLScrollBar();
    offset = offset + 1
  end
  scrollBar:SetValue(floor(offset * 32 + 0.5));
  -- print(offset, scrollBar:GetValue());
end

-- ====================================================================
--  Drag and drop functions
-- ====================================================================
function Torta.gui:clShowDropHandlers()
  for _, button in pairs(self.clListButtons) do
    button.topDropHandler:Show();
    button.bottomDropHandler:Show();
    UIFrameFadeOut(button.text, 0.3, button.text:GetAlpha(), 0.5);
    UIFrameFadeOut(button.check, 0.3, button.check:GetAlpha(), 0.5);
  end
end

function Torta.gui:clHideDropHandlers()
  for _, button in pairs(self.clListButtons) do
    button.topDropHandler:Hide();
    button.bottomDropHandler:Hide();
    UIFrameFadeIn(button.text, 0.3, button.text:GetAlpha(), 1);
    UIFrameFadeIn(button.check, 0.3, button.check:GetAlpha(), 1);
  end
end

function Torta.gui:clTaskDragStart(taskButton)
  PlaySound("INTERFACESOUND_CURSORGRABOBJECT");
  local text = taskButton.text:GetText();
  local stat = taskButton.check:GetChecked();
  TortaDragThingy.text:SetText(text);
  TortaDragThingy.check:SetChecked(stat);
  TortaDragThingy:Show();
  UIFrameFadeIn(TortaDragThingy, 0.25, 0, 1);
  self:clShowDropHandlers();
  self.dragSrcID = taskButton:GetID();
end

function Torta.gui:clTaskDragStop()
  PlaySound("INTERFACESOUND_CURSORDROPOBJECT");
  TortaDragThingy:Hide();
  if GetMouseFocus().tortaDropType == "CLTASK" then
    Torta.cl3:MoveTask(self.dragSrcID, GetMouseFocus().destLoc);
    GetMouseFocus():Hide(); -- If this line isn't here, the drop handler under the mouse will fail to hide
  end
  self:clHideDropHandlers();
end


function Torta.gui:clexShowDropHandlers()
  for _, button in pairs(self.exListButtons) do
    button.topDropHandler:Show();
    button.bottomDropHandler:Show();
    UIFrameFadeOut(button.text, 0.3, button.text:GetAlpha(), 0.5);
  end
end

function Torta.gui:clexHideDropHandlers()
  for _, button in pairs(self.exListButtons) do
    button.topDropHandler:Hide();
    button.bottomDropHandler:Hide();
    UIFrameFadeIn(button.text, 0.3, button.text:GetAlpha(), 1);
  end
end

function Torta.gui:clListDragStart(listButton)
  PlaySound("INTERFACESOUND_CURSORGRABOBJECT");
  local text = listButton.text:GetText();
  TortaCLListDragThingy.text:SetText(text);
  TortaCLListDragThingy:Show();
  UIFrameFadeIn(TortaCLListDragThingy, 0.25, 0, 1);
  self:clexShowDropHandlers();
  self.dragSrcID = listButton:GetID();
end

function Torta.gui:clListDragStop()
  PlaySound("INTERFACESOUND_CURSORDROPOBJECT");
  TortaCLListDragThingy:Hide();
  if GetMouseFocus().tortaDropType == "CLLIST" then
    Torta.cl3:MoveList(self.dragSrcID, GetMouseFocus().destLoc);
    GetMouseFocus():Hide(); -- If this line isn't here, the drop handler under the mouse will fail to hide
  end
  self:clexHideDropHandlers();
end

-- Note:
-- Add all UI management functions ABOVE!

-- ====================================================================
-- Dropdown menu stuff
-- ====================================================================
local clExplorerDropdownInfo = {
  {
    text = "Tortoise Explorer",
    isTitle = true
  },
  {
    text = "Create a new list",
    func = function() Torta.cl3:CreateList(); end
  },
  {
    text = "Profiles... (NYI)",
    func = function() end
  },
  {
    text = "",
    notClickable = true
  },
  {
    text = "Close",
    func = function() TortaDropDownMenuFrame:Hide(); end
  }
}
local clListDropdownInfo = {
  {
    text = "<will get replaced by list name>",
    isTitle = true
  },
  {
    text = "Rename this list",
    func = function() Torta.cl3:RenameList(Torta.gui.selectedExplorerButton:GetID()); end
  },
  {
    text = "Delete this list",
    func = function() Torta.cl3:DeleteList(Torta.gui.selectedExplorerButton:GetID()); end
  },
  {
    text = "",
    notClickable = true
  },
  {
    text = "Create a new list",
    func = function() Torta.cl3:CreateList(); end,
  },
  {
    text = "Profiles... (NYI)",
    func = function() end
  },
  {
    text = "",
    notClickable = true
  },
  {
    text = "Close",
    func = function() TortaDropDownMenuFrame:Hide(); end
  }
}
local dropdown = CreateFrame("Frame", "TortaDropDownMenuFrame", explorerWindow, "UIDropDownMenuTemplate");

function Torta.gui:clShowButtonDropdown(button)
  assert(button);
  self.selectedExplorerButton = button;
  clListDropdownInfo[1].text = button.text:GetText();
  EasyMenu(clListDropdownInfo, dropdown, "cursor", 0, 0, "MENU", 3);
end


function Torta.gui:clShowExplorerDropdownMenu()
  EasyMenu(clExplorerDropdownInfo, dropdown, "cursor", 0, 0, "MENU", 3);
end

-- ====================================================================
--  Dialogs
-- ====================================================================
StaticPopupDialogs["TORTA_CLRENAMELIST"] = {
  text = "Please enter the new name for the list currently named \"|cffffd000%s|r\":",
  OnAccept = function(frame)
    local newName = getglobal(frame:GetName() .. "EditBox"):GetText();
    Torta.cl3:RenameList(Torta.gui.selectedExplorerButton:GetID(), newName);
    frame:Hide();
  end,
  EditBoxOnEnterPressed = function(editBox)
    local newName = editBox:GetText();
    Torta.cl3:RenameList(Torta.gui.selectedExplorerButton:GetID(), newName);
    editBox:GetParent():Hide();
  end,
  EditBoxOnEscapePressed = function(self)
    self:GetParent():Hide();
  end,
  button1 = ACCEPT,
  button2 = CANCEL,
  whileDead = 1,
  timeout = 30,
  hasEditBox = 1,
  hideOnEscape = 1,
}
StaticPopupDialogs["TORTA_CLDELETELIST"] = {
  text = "Are you ABSOLUTELY sure you want to delete the list \"|cffffd000%s|r\"?",
  button1 = YES,
  button2 = NO,
  showAlert = true,
  OnAccept = function(self, data)
    Torta.cl3:DeleteList(data, true);
  end,
  whileDead = 1,
  timeout = 30,
  hideOnEscape = 1
}
--  Dialogs end here
-- ====================================================================

--

-- ====================================================================
--  Misc. UI init code
-- ====================================================================
FauxScrollFrame_SetOffset(explorerScrollBar, 0);
FauxScrollFrame_SetOffset(clScrollBar, 0);

mainWindow:SetMinResize(820, 450);
PanelTemplates_SetNumTabs(mainWindow, 3);
Torta.gui:OnTabClicked(1);
